#!/usr/bin/python3

# dCache Kerberos principal management
# Arguments:
#     add/delete
#     username

# System definitions.
import os
import sys
import re

# Extra installed module
import kadmin

keytab = "/home/dcache/private/dcache-usermgmt.keytab"
adminprincipal = "dcache/usermgmt"

# Get arguments and check. Need to have two arguments.
if len(sys.argv) != 3:
    print("Add or remove user principals in kerberos.")
    print("Arguments: <action> <username>")
    print("")
    print("Action is either add or delete.")
    print("Username should match s_AAABBB usernames.")
    sys.exit(1)

action=sys.argv[1]
user = sys.argv[2]

# Check for the user. Should look like a Swestore user.
if not (re.match("^s_", user)):
    print("Not a Swestore user name.")
    sys.exit(1)

# Kadmin init or "login"
try:
    kadm = kadmin.init_with_keytab(adminprincipal, keytab)
    #kadm = kadmin.init_with_ccache(adminprincipal, "/tmp/krb5cc_0")
    #kadm = kadmin.init_with_password(adminprincipal, "Foobar")
except Exception as e:
    print("Init failed:", e)
    sys.exit(1)

# Get the user principal.
# Will be None if no-such-user and this would not be an error.
try:
    princ = kadm.getprinc(user)
except Exception as e:
    print("Getprinc failed:", e)
    sys.exit(1)

# Run the actions
if (action == "add"):
    if (princ is None):
        # User does not exist, add it.
        try:
            kadm.addprinc(user, None)
            print("User " + user + " added.")
        except Exception as e:
            print("Addprinc failed:", e)
            sys.exit(1)
    else:
        print("User", user, "already exists.")
        sys.exit(1)

elif(action == "delete"):
    if (princ is None):
        print("No such user in database:", user)
        sys.exit(1)
    else:
        # User exists, try to remove it.
        try:
            kadm.delprinc(user)
            print("User " + user + " deleted.")
            sys.exit(0)
        except Exception as e:
            print("Delprinc failed:", e)
            sys.exit(1)

else:
    # The first argument should be add or delete
    print("Not a valid action: \"" + action + "\". Should be \"add\" or \"delete\".")
    sys.exit(1)
        
# Finish.
sys.exit(0)
