#!/usr/bin/python
#! -*- coding: utf-8 -*-
# Likely obsolete, may be removed in the future
import irods
import ldap
import ldap.modlist as modlist
import logging
import sys
import supr
import smtplib

from email.mime.text import MIMEText
import settings

from supr_common import (sendMail,
			 setup_log)

class WEEKLY_CLEANUP:
	def __init__(self, supr_persons, supr_groups):

		# Get Logging File Handler
		self.logger = setup_log(self.__class__.__name__, settings.DELETE_LOG_FILE)

		self.l            = None
		self.conn         = None
		self.supr_persons = supr_persons
		self.supr_groups  = supr_groups

		# Variables needed for calculating number of projects and persons created 
		# that are used to sendMail to Admins
		self.del_ldap_proj_cnt        = 0
		self.del_irods_proj_cnt       = 0

		self.del_ldap_pers_cnt        = 0
		self.del_irods_pers_cnt       = 0

		self.err_ldap_proj_cnt        = 0
		self.err_ldap_pers_cnt        = 0
		self.err_ldap_mjr_cnt         = 0
		self.err_irods_mjr_cnt        = 0
		self.ldap_msg_text            = ""
		self.irods_msg_text           = ""

		# Variables needed for Message Text of projects and persons created 
		# that are used to sendMail to Admins
		self.DEL_LDAP_PROJ_MAIL   = "The Following Projects have been deleted from LDAP on " + settings.today + "\n" 
		self.DEL_LDAP_PROJ_MAIL  += "----------------------------------------------------------- \n"

		self.DEL_LDAP_PERS_MAIL   = "The Following Persons have been deleted from LDAP on " + settings.today + "\n" 
		self.DEL_LDAP_PERS_MAIL  += "----------------------------------------------------------- \n"

		self.DEL_IRODS_PROJ_MAIL  = "The Following Projects have been deleted from SUPR on " + settings.today + "\n" 
		self.DEL_IRODS_PROJ_MAIL += "----------------------------------------------------------- \n"

		self.DEL_IRODS_PERS_MAIL  = "The Following Persons have been deleted from SUPR on " + settings.today 
		self.DEL_IRODS_PERS_MAIL += "\n" + "----------------------------------------------------- \n"

		self.ERR_LDAP_PROJ_MAIL   = "Error in deleting the following Projects from LDAP on " + settings.today + "\n" 
		self.ERR_LDAP_PROJ_MAIL  += "------------------------------------------------------- \n"

		self.ERR_LDAP_PERS_MAIL   = "Error in deleting the following Persons from LDAP on " + settings.today + "\n" 
		self.ERR_LDAP_PERS_MAIL  += "------------------------------------------------------ \n"

		self.ERR_LDAP_MJR_MAIL    = "Major Error Occured on " + settings.today + "\n" 
		self.ERR_LDAP_MJR_MAIL   += "---------------------------------- \n"

		self.ERR_IRODS_MJR_MAIL   = "Major Error Occured on " + settings.today + "\n" 
		self.ERR_IRODS_MJR_MAIL  += "---------------------------------- \n"


		# LDAP search scope and other variables
		self.searchScope = ldap.SCOPE_SUBTREE

		# Connect to LDAP using python-LDAP
		try:
			ldap.set_option(ldap.OPT_X_TLS_CACERTFILE,settings.TLS_CACERTFILE)
			ldap.set_option(ldap.OPT_X_TLS_CERTFILE,settings.TLS_CERTFILE)
			ldap.set_option(ldap.OPT_X_TLS_KEYFILE,settings.TLS_KEYFILE)
			ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT,ldap.OPT_X_TLS_DEMAND)

			self.l = ldap.initialize(settings.LDAP_HOST)
			#l = ldap.open(settings.LDAP_HOST)
			self.l.simple_bind(settings.LDAP_ADMIN, settings.LDAP_PASSWORD)

		except ldap.LDAPError as le:
			self.logger.error("LDAP Connection Error - %s", le)
			self.ERR_LDAP_MJR_MAIL += "Error occured in LDAP Connection - " + str(le) + "\n"
			self.err_ldap_mjr_cnt  += 1

			if self.l:
				self.l.unbind()
			sys.exit(1)

		# Connect to iRODS using pyRODS
		try:
			self.envVal = irods.obfiGetEnvKey()
			self.obi_password = irods.obfiEncode(settings.IRODS_ADMIN_PWD, self.envVal)
			# Connection to a server with the default values
			self.conn, self.errMsg = irods.rcConnect(settings.IRODS_HOST, settings.IRODS_PORT, settings.IRODS_ADMIN_USER, settings.IRODS_ZONE)
			if self.conn:
				self.status = irods.clientLoginWithObfPassword(self.conn, self.obi_password)
				self.irods_projects = irods.irodsCollection(self.conn, settings.IRODS_DIR)
				self.userGroup = irods.getGroup(self.conn, "users")
			else:
				raise irods.IrodsException(self.errMsg.status)

		except irods.IrodsException as ie:
			self.logger.error("Error during iRODS Connection: %s", ie)
			self.ERR_IRODS_MJR_MAIL += "Error occured in iRODS Connection - " + str(ie) + "\n"
			self.err_irods_mjr_cnt  += 1

			if self.conn:
				self.conn.disconnect()
			sys.exit(1)

		self.deletePersons()
		self.deleteProjects()

		# Call to sendMail() Function for LDAP
		if self.err_ldap_mjr_cnt:
			self.ldap_msg_text += str(self.ERR_LDAP_MJR_MAIL) + "\n"
		if self.err_ldap_proj_cnt:
			self.ldap_msg_text += str(self.ERR_LDAP_PROJ_MAIL) + "\n"
		if self.err_ldap_pers_cnt:
			self.ldap_msg_text += str(self.ERR_LDAP_PERS_MAIL) + "\n"

		if self.del_ldap_proj_cnt:
			self.ldap_msg_text += str(self.DEL_LDAP_PROJ_MAIL) + "\n"
		if self.del_ldap_pers_cnt:
			self.ldap_msg_text += str(self.DEL_LDAP_PERS_MAIL) + "\n"

		if self.ldap_msg_text:
			sendMail(self.ldap_msg_text, settings.FROM_ADDRS, settings.DCACHE_ADMIN_MAIL, settings.LDAP_DEL_SUBJECT)


		# Call to sendMail() Function for iRODS
		if self.err_irods_mjr_cnt:
			self.irods_msg_text += str(self.ERR_IRODS_MJR_MAIL) + "\n"

		if self.del_irods_proj_cnt:
			self.irods_msg_text += str(self.DEL_IRODS_PROJ_MAIL) + "\n"
		if self.del_irods_pers_cnt:
			self.irods_msg_text += str(self.DEL_IRODS_PERS_MAIL) + "\n"

		if self.irods_msg_text:
			sendMail(self.irods_msg_text, settings.FROM_ADDRS, settings.IRODS_ADMIN_MAIL, settings.IRODS_DEL_SUBJECT)


	# Delete Persons from LDAP and iRODS that are not in SUPR
	def deletePersons(self):

		try:
			# Get All Persons from LDAP
			filter = "uidNumber=*"
			retrievedAttrs = ['description', 'uid','uidNumber']
			ldap_result_id = self.l.search(settings.peopleDN, self.searchScope, filter, retrievedAttrs)
			result_type, ldap_persons = self.l.result(ldap_result_id)

		except ldap.LDAPError as le:
			self.logger.error("LDAP Error in deletePersons Module search - %s", le)
			self.ERR_LDAP_MJR_MAIL += "Error occured in deletePersons Module search LDAP - " + str(le) + "\n"
			self.err_ldap_mjr_cnt  += 1

		# Loop through LDAP results
		for person in ldap_persons:

			# Check if the ldap Person exists in SUPR
			if person[1]['description'][0] in self.supr_persons:
				self.logger.info("SUPR Person ID %s exists. Need not delete from LDAP", person[1]['description'][0])
			else:
				try:
					self.l.delete_s(person[0])
					self.logger.info("SUPR Person ID %s doesnot exist. Deleted from LDAP", person[1]['description'][0])

					self.DEL_LDAP_PERS_MAIL += "SUPR ID :: " + str(person[1]['description'][0]) + "\t" + "username(uid) :: " + str(person[1]['uid'][0])
					self.del_ldap_pers_cnt  += 1
				except ldap.LDAPError as le:
					self.logger.error("LDAP Error in deletePersons Module delete for %s :: %s", str(person[1]['uidNumber'][0]), le)
					self.ERR_LDAP_PERS_MAIL += "SUPR ID :: " + str(person[1]['description'][0]) + "\t" + "username(uid) :: " + str(person[1]['uid'][0])
					self.err_ldap_pers_cnt  += 1

				try:
					# write code to disable person from iRODS
					if self.conn and irods.getUser(self.conn, str(person[1]['uid'][0])):
						# disable user from iRODS
						self.logger.info("SUPR Person ID %s doesnot exist. Disable from iRODS", person[1]['description'][0])

						self.DEL_IRODS_PERS_MAIL += "SUPR ID :: " + str(person[1]['description'][0]) + "\t" + "username(uid) :: " + str(person[1]['uid'][0])
						self.del_irods_pers_cnt  += 1
				except irods.IrodsException as ie:
					self.logger.error("Error during iRODS Connection: %s", ie)


	# Delete Projects from LDAP and iRODS that are not in SUPR
	def deleteProjects(self):

		try:
			# Get All Projects from LDAP
			filter = "gidNumber=*"
			retrievedAttrs = ['description','gidNumber', 'cn']
			ldap_result_id = self.l.search(settings.groupsDN, self.searchScope, filter, retrievedAttrs)
			result_type, ldap_groups = self.l.result(ldap_result_id)

		except ldap.LDAPError as le:
			self.logger.error("LDAP Error in deleteProjects Module search - %s", le)
			self.ERR_LDAP_MJR_MAIL += "Error occured in deleteProjects Module search LDAP - " + str(le) + "\n"
			self.err_ldap_mjr_cnt  += 1

		for group in ldap_groups:
			# Checking for projects in SUPR
			proj_supr_id    = str(group[1]['description'][0])
			proj_name       = str(group[1]['cn'][0])

			if group[0].find("gidNumber=5") < 0:

				if proj_supr_id in self.supr_groups:
					self.logger.info("SUPR Project ID %s exists. Not needed to delete from LDAP", group[1]['description'][0])
				else:
					try:
						self.l.delete_s(group[0])
						self.logger.info("SUPR Project ID %s doesnot exist. Deleted from LDAP", group[1]['description'][0])
						self.DEL_LDAP_PROJ_MAIL += "SUPR ID :: " + proj_supr_id + "\t" + "Project Name :: " + proj_name
						self.del_ldap_proj_cnt  += 1

					except ldap.LDAPError as le:
						self.logger.error("LDAP Error in deleteProjects Module delete for %s :: %s", str(group[1]['gidNumber'][0]), le)
						self.ERR_LDAP_PROJ_MAIL += "SUPR ID :: " + proj_supr_id + "\t" + "Project Name :: " + proj_name
						self.err_ldap_proj_cnt  += 1

					try:
						# write code to disable project from iRODS
						#if proj_name in c.getSubCollections():
							# disable project
						# write code to disable group from iRODS
						if self.conn and irods.getGroup(self.conn, proj_name):
							# disable group
							self.DEL_IRODS_PROJ_MAIL += "SUPR ID :: " + proj_supr_id + "\t" + "Project Name :: " + proj_name
							self.del_irods_proj_cnt  += 1
					except irods.IrodsException as ie:
						self.logger.error("Error during iRODS Connection: %s", ie)


			# Checking for Projects created for each Person
			else:
				if proj_supr_id in self.supr_persons:
					self.logger.info("LDAP Person ID %s exists. Not needed to delete from LDAP", group[1]['description'][0])
				else:
					try:
						self.l.delete_s(group[0])
						self.logger.info("LDAP Person ID %s doesnot exist. Deleted from LDAP", group[1]['description'][0])
					except ldap.LDAPError as le:
						self.logger.error("LDAP Error in deleteProjects Module - %s", le)

# Compose query from the below options
params = {'full_person_data': '1','resource_centre_id': settings.resource_centre_id}

# Search in SUPR
try:
	# Prepare a SUPR connection object
	supr_connection = supr.SUPR(user = settings.SUPR_API_USER_NAME, 
				    password = settings.SUPR_API_PASSWORD,
                                    base_url = settings.SUPR_BASE_URL)

	# Get All Projects
	res = supr_connection.get("project/search/", params = params)

except supr.SUPRHTTPError as e:
	# Logging Errors from SUPR
	logger.error("HTTP error from SUPR - %s" %e.text)
	sys.exit(1)

# Loop through the results from the search in SUPR
supr_groups  = []
supr_persons = []
for p in res.matches:

	# Add Projects to SUPR
	if str(p.id) not in supr_groups:
		supr_groups.append(str(p.id))

	for m in p.members:
		if str(m.id) not in supr_persons:
			supr_persons.append(str(m.id))

# Instantiating and Calling the class
if supr_groups or supr_persons:
	WEEKLY_CLEANUP(supr_persons, supr_groups)

