#!/usr/bin/python
#! -*- coding: utf-8 -*-
import logging
import sys
import supr
import smtplib
import os

from email.mime.text import MIMEText
from irods import *
from settings import *
from subprocess import call, check_output

# logging configuration
logging.basicConfig(filename = IRODS_LOG_FILE, 
		    filemode = LOG_FILE_MODE,
		    format = LOG_FILE_FORMAT,
		    datefmt = LOG_FILE_DATEFORMAT,
		    level = logging.DEBUG)

# Compose query from the below options
params = {'full_person_data': '1','resource_centre_id': resource_centre_id, 'resource_id': irods_resource_id}

# Search in SUPR
try:
	# Prepare a SUPR connection object
	supr_connection = supr.SUPR(user = SUPR_API_USER_NAME, password = SUPR_API_PASSWORD)

	# Get All Projects
	res = supr_connection.get("project/search/", params = params)
except supr.SUPRHTTPError as e:
	# Logging Errors from SUPR
	logging.error("HTTP error from SUPR - %s" %e.text)
	sys.exit(1)

try:
	envVal = obfiGetEnvKey()
	obi_password = obfiEncode(IRODS_ADMIN_PWD, envVal)

	# Connection to a server with the default values
	conn, errMsg = rcConnect(IRODS_HOST, IRODS_PORT, IRODS_ADMIN_USER, IRODS_ZONE)
	status = clientLoginWithObfPassword(conn, obi_password)

	c = irodsCollection(conn, IRODS_DIR)
	userGroup = getGroup(conn, "users")

	for p in res.matches:

		proj_name = ((p.directory_name).encode('utf-8')).lower()
		group = getGroup(conn, proj_name)

		for m in p.members:

			user = getUser(conn,str(m.centre_person_id))
			if group:
				if user and (user.getId() in (groupuser.getId() for groupuser in group.getMembers())):
					group.rmUser(str(m.centre_person_id))
					logging.info(str(m.centre_person_id) + " user deleted from " + proj_name + " group")
				else:
					logging.info(str(m.centre_person_id) + " user already deleted from " + proj_name + " group")

			if userGroup:
				if user and (user.getId() in (groupuser.getId() for groupuser in userGroup.getMembers())):
					userGroup.rmUser(str(m.centre_person_id))
					logging.info(str(m.centre_person_id) + " user deleted from user group")
				else:
					logging.info(str(m.centre_person_id) + " user already deleted from users group")

			if user:
				deleteUser(conn, str(m.centre_person_id))
				logging.info(str(m.centre_person_id) + " user deleted.")
			else:
				logging.info(str(m.centre_person_id) + " user already deleted")

		if getGroup(conn, proj_name):
			group = deleteGroup(conn, proj_name)
			logging.info(proj_name + " group deleted")
		else:
			logging.info(proj_name + " group already deleted")

		if proj_name in c.getSubCollections():
			c.deleteCollection(proj_name)
			logging.info(IRODS_DIR + proj_name + " directory deleted")
		else:
			logging.info(IRODS_DIR + proj_name + " directory already deleted")

except Exception as e:
	logging.error("Error during Execution: %s", e)
finally:
	conn.disconnect()
